#ifndef VALID_SEQ_H
#define VALID_SEQ_h

#include <map>
#include <set>
#include <string>
#include <cstring>
#include <fstream>
#include <iostream>
#include <cassert>
#include <cstdlib>

using namespace std;

struct range {
	unsigned int begin;
	unsigned int end;
	
	range() :
		begin(0), end(0) {}
	range(unsigned int b, unsigned int e):
		begin(b), end(e) {}
		
	bool operator ==(const range &other) {
		if( (begin <= other.begin) && (end >= other.end) )
			return true;
		return false;
	}
	
	bool operator< (const range &other) const {
		return (begin < other.begin) && (end < other.end);
	}
};

class valid_seq {
	public:
		valid_seq();
		// If you wish to pass in a file name, you can do that here.
		valid_seq(char* fn);
		
		/*!
		 * store will take the sequences in the given file, fn, and store them into memory, more
		 * specifically, into the map-set valid_seqs.  
		 * In order to be read, the file should be in the format:
		 *		chrom_name <tab> starting position
		 * In the header of the file, two other numbers must be included:
		 * 		original_sequence_length <tab> length_of_region
		 * ie: 	if the original sequences were 30 bp in length, and a region of 50 bp should be
		 *		considered, include 30 <tab> 50.  This will place the 30 bp in the middle of the
		 * 		50 surrounding bp.
		 *
		 * @param fn - the file name of the sequences that are valid
		 */
		void store(const char* fn);
		
		/*!
		 * is_valid will determine if the following position is valid.  Genome.h already has
		 * implemented a function to return the chromosome name and position in one string.
		 * is_valid will parse through the string (using strtok) and remove the chromosome name
		 * and position.  Then, it will check against the existing map-set valid_seqs and determine
		 * if this sequence lies within the set.
		 *
		 * @param chr_pos - the string (obtained from Genome) to be parsed for chr name and pos
		 * @param size - the size of the sequence--used for obtaining the top end of this sequence.
		 */
		bool is_valid(string &chr_pos, int size);
		
		/*!
		 * For the sake of simplicity, we will assume that, when using the ordered random method, 
		 * there is only one chromosome.  Otherwise, we would have to deal a lot with from relative
		 * to absolute position and visa versa.
		 */
		unsigned int get_next_rand(unsigned int &num);
	private:
		/* valid_seqs is a map of strings (representing chromosome name) mapped to a set of
		 * unsigned ints (representing the valid positions)
		 */
		map<string,set<range> > valid_seqs;
		
		int seq_size;
		int range_size;
		unsigned int total_seqs;
};

#endif //VALID_SEQ_H
